#include "Parsodus/lrtables/LALR1Itemset.h"

#include <cassert>

namespace pds {
namespace lr {

LALR1Itemset::LALR1Itemset() : LR1ItemsetBase<LALR1Itemset>()
{}

LALR1Itemset::LALR1Itemset(std::shared_ptr<Rule> start) : LR1ItemsetBase<LALR1Itemset>(start)
{}

bool LALR1Itemset::canMerge(const LALR1Itemset& rhs) const {
    if (rhs.m_items.size() != m_items.size()) {
        return false;
    }

    for (auto& rhsIt : rhs.m_items) {
        bool ok = false;
        for (auto& it : m_items) {
            if (it.rule == rhsIt.rule && it.dotIdx == rhsIt.dotIdx) {
                ok = true;
                break;
            }
        }
        if (!ok)
            return false;
    }

    return true;
}

bool LALR1Itemset::merge(const LALR1Itemset& rhs) {
    bool change = false;
    for (auto& toMerge : rhs.m_items) {
        bool found = false;
        for (auto& it : m_items) {
            if (it.rule == toMerge.rule && it.dotIdx == toMerge.dotIdx) {
                found = true;
                for (auto& la : toMerge.lookaheads) {
                    if (!it.lookaheads.count(la)) {
                        change = true;
                        it.lookaheads.insert(la);
                    }
                }
                break;
            }
        }
        assert(found);
    }
    return change;
}

} /* lr  */ 
} /* pds  */ 
