#include "Parsodus/util/followset.h"
#include "Parsodus/util/symbols.h"
#include "Parsodus/grammar.h"

namespace pds {
namespace util {

FollowSet::FollowSet(const Grammar& g, const FirstSet& first)
{
    m_follow[EXTENDED_START].insert(EOF_PLACEHOLDER);

    bool changes = true;

    auto update = [&changes, this](std::string head, auto s) {
        s.erase("");
        for (std::string elem : s) {
            if (!m_follow[head].count(elem)) {
                changes = true;
                m_follow[head].insert(s.begin(), s.end());
                return;
            }
        }
    };

    while (changes) {
        changes = false;
        for (const auto& rule : g.rules) {
            const std::string& head = rule->head;
            auto it = rule->tail.begin(); //< Keep track of the start of the rest of the tail
            it++; //< The 'rest' of the tail
            for (std::size_t i = 0; i < rule->tail.size(); i++, it++) {
                if (g.variables.count(rule->tail[i])) {
                    std::set<std::string> restFirst = first(std::vector<std::string>(it, rule->tail.end()));
                    if (i == rule->tail.size() - 1 || restFirst.count("")) {
                        update(rule->tail[i], m_follow[head]);
                    }
                    if (i < rule->tail.size() - 1) {
                        update(rule->tail[i], restFirst);
                    }
                }
            }
        }
    }
}

std::set<std::string> FollowSet::operator()(std::string key) const {
    auto tmp = m_follow.find(key);
    if (tmp == m_follow.end())
        return {};
    return tmp->second;
}

} /* util */
} /* pds */
